@@ -27,6 +27,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -38,6 +39,8 @@
 #include "net.h"
 #include "color.h"
 #include "types.h"
+#include "utils.h"
+#include "../pixmaps/xconvers.xpm"
 
 /************************ DEFINES AND GLOBAL VARIABLES ***********************/
 
@@ -47,10 +50,10 @@
 extern GtkWidget *mainwindow;
 gchar *preferencesdir, *logfilename;
 gpointer cvhost, cvport;
-GdkColor white, grey, red, pink, color[10];
 GdkFont *textfont;
 GtkWidget *preferencesdialog;
-gint rxmonitor;
+GtkButton *selectedcolorbutton;
+gint rxmonitor, buttonnumber = -1, mousebutton = 0;
 gboolean connected = FALSE, locked = FALSE;
 GList *txhistory, *hostnamehistory, *porthistory;
 FILE *logfp;
@@ -67,36 +70,12 @@ preferencestype preferences;
 
 void on_mainwindow_show(GtkWidget *widget, gpointer user_data)
 {
-  GtkWidget *menuclose, *maintext, *mainstatusbar, *mainscrolledwindow;
-  GString *greeting;
-  GtkStyle *maintextstyle;
+  GtkWidget *menuclose, *mainstatusbar, *mainscrolledwindow;
+  GString *greeting = g_string_new("");
 
-  dircheck();
-  loadpreferences();
-  loadhistory();
   menuclose = lookup_widget(mainwindow, "close");
   gtk_widget_set_sensitive(menuclose, 0);
-  gdk_colormap_alloc_color(gdk_colormap_get_system(), 
-    &preferences.background, 1, 1);
-  gdk_color_parse("white", &white);
-  gdk_color_parse("red", &red);
-  gdk_color_parse("pink", &pink);
-  gdk_color_parse("light grey", &grey);
-  gdk_color_parse("yellow", &color[0]);
-  gdk_color_parse("green", &color[1]);
-  gdk_color_parse("blue", &color[2]);
-  gdk_color_parse("magenta", &color[3]);
-  gdk_color_parse("orange", &color[4]);
-  gdk_color_parse("cyan", &color[5]);
-  gdk_color_parse("purple", &color[6]);
-  gdk_color_parse("DarkKhaki", &color[7]);
-  gdk_color_parse("DarkCyan", &color[8]);
-  gdk_color_parse("DeepPink", &color[9]);
-  maintext = lookup_widget(mainwindow, "maintext");
-  maintextstyle = gtk_style_new();
-  maintextstyle->base[GTK_STATE_NORMAL] = preferences.background;
-  maintextstyle->base[GTK_STATE_INSENSITIVE] = preferences.background;
-  gtk_widget_set_style(maintext, maintextstyle);
+
   textfont = gdk_font_load(preferences.font->str);
   if (!preferences.menu) widget_visible("mainmenubar", 0);
   if (!preferences.statusbar) widget_visible("mainstatusbar", 0);
@@ -106,8 +85,7 @@ void on_mainwindow_show(GtkWidget *widget, gpointer user_data)
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(mainscrolledwindow), 
       GTK_POLICY_NEVER, GTK_POLICY_NEVER);
   };
-  greeting = g_string_new("Welcome to ");
-  g_string_append(greeting, PACKAGE);
+  g_string_sprintf(greeting, _("Welcome to %s"), PACKAGE);
   mainstatusbar = lookup_widget (mainwindow, "mainstatusbar");
   gtk_statusbar_push (GTK_STATUSBAR(mainstatusbar), 1, greeting->str);
   if (!preferences.statusbar)
@@ -228,13 +206,11 @@ void on_open_activate(GtkMenuItem *menuitem, gpointer user_data)
 void on_close_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
   GtkWidget *closedialog, *closelabel;
-  GString *labeltext;
+  GString *labeltext = g_string_new("");
 
   closedialog = create_closedialog ();
   closelabel = lookup_widget (closedialog, "closelabel");
-  labeltext = g_string_new("Close connection to ");
-  g_string_append(labeltext, cvhost);
-  g_string_append(labeltext," ?");
+  g_string_sprintf(labeltext, _("Close connection to %s ?"), (gchar *)cvhost);
   gtk_label_set_text(GTK_LABEL(closelabel), labeltext->str);
   g_string_free(labeltext, TRUE);
   gtk_widget_set_sensitive(mainwindow, 0);
@@ -259,8 +235,16 @@ void on_preferences_activate(GtkMenuItem *menuitem, gpointer user_data)
 void on_about_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
   GtkWidget *aboutdialog;
+  GtkStyle *aboutstyle;
+  GdkBitmap *bitmap;
+  GdkPixmap *icon;
 
   aboutdialog = create_aboutdialog();
+  gtk_widget_realize(aboutdialog);
+  aboutstyle = gtk_widget_get_style(aboutdialog);
+  icon = gdk_pixmap_create_from_xpm_d(aboutdialog->window, &bitmap, &aboutstyle->bg[GTK_STATE_NORMAL], xconvers_xpm);
+  gdk_window_set_icon(aboutdialog->window, NULL, icon, bitmap);
+  gdk_window_set_icon_name(aboutdialog->window, _("about"));
   gtk_widget_show(aboutdialog);
 }
 
@@ -280,10 +264,8 @@ void on_opendialog_show(GtkWidget *widget, gpointer user_data)
   opendialog = gtk_widget_get_toplevel(widget);
   hostnamecombo = lookup_widget(opendialog, "hostnamecombo");
   portcombo = lookup_widget(opendialog, "portcombo");
-  if (hostnamehistory) gtk_combo_set_popdown_strings(GTK_COMBO(hostnamecombo),
-    hostnamehistory);
-  if (porthistory) gtk_combo_set_popdown_strings(GTK_COMBO(portcombo),
-    porthistory);
+  if (hostnamehistory) gtk_combo_set_popdown_strings(GTK_COMBO(hostnamecombo), hostnamehistory);
+  if (porthistory) gtk_combo_set_popdown_strings(GTK_COMBO(portcombo), porthistory);
   gtk_widget_set_sensitive(mainwindow, 0);
 }
 
@@ -307,15 +289,12 @@ void on_openOKbutton_clicked(GtkButton *button, gpointer user_data)
   portcombo = lookup_widget(opendialog, "portcombo");
   hostnamecomboentry = GTK_COMBO(hostnamecombo)->entry;
   portcomboentry = GTK_COMBO(portcombo)->entry;
-  strcpy(cvhost, gtk_editable_get_chars(GTK_EDITABLE(hostnamecomboentry), 0,
-    -1));
+  strcpy(cvhost, gtk_editable_get_chars(GTK_EDITABLE(hostnamecomboentry), 0, -1));
   strcpy(cvport, gtk_editable_get_chars(GTK_EDITABLE(portcomboentry), 0, -1));
   if (!strcmp(cvhost, "")) strcpy(cvhost, "localhost");
   if (!strcmp(cvport, "")) strcpy(cvport, "3600");
-  node = g_list_find_custom(hostnamehistory, g_strdup(cvhost),
-    (GCompareFunc)g_strncasecmp);
-  if (!node) hostnamehistory = g_list_prepend(hostnamehistory,
-    g_strdup(cvhost));
+  node = g_list_find_custom(hostnamehistory, g_strdup(cvhost), (GCompareFunc)g_strncasecmp);
+  if (!node) hostnamehistory = g_list_prepend(hostnamehistory, g_strdup(cvhost));
   else
   {
     if (g_list_position(hostnamehistory, node) != 0)
@@ -325,10 +304,8 @@ void on_openOKbutton_clicked(GtkButton *button, gpointer user_data)
     }
   }
   if (g_list_length(hostnamehistory) > HOSTNAMEHISTORY)
-    hostnamehistory = g_list_remove(hostnamehistory,
-      g_list_last(hostnamehistory)->data);
-  node = g_list_find_custom(porthistory, g_strdup(cvport),
-    (GCompareFunc)g_strncasecmp);
+    hostnamehistory = g_list_remove(hostnamehistory, g_list_last(hostnamehistory)->data);
+  node = g_list_find_custom(porthistory, g_strdup(cvport), (GCompareFunc)g_strncasecmp);
   if (!node) porthistory = g_list_prepend(porthistory, g_strdup(cvport));
   else
   {
@@ -409,59 +386,118 @@ void on_closedialog_destroy(GtkObject *object, gpointer user_data)
 /**************************** PREFERENCES DIALOG *****************************/
 
 /*
- * Set the title of the preferences window and check or uncheck autologin and 
- * logging boxes. The preferences struct is checked to see if the name and 
- * channel strings are assigned. If so, the entries will be filled in. The 
+ * Check or uncheck autologin and logging boxes. 
+ * The preferences struct is checked to see if the name, channel
+ * and command strings are assigned. If so, the entries will be filled in. The 
  * background button gets the same color as the main text window and the font
  * entry is filled.
  */
  
 void on_preferencesdialog_show(GtkWidget *widget, gpointer user_data)
 {
-  GtkStyle *style;
   GtkWidget *loggingcheckbutton, *autologincheckbutton, *autologinnamelabel, 
-    *autologinchannellabel, *autologinnameentry, *autologinchannelentry,
-    *backgroundcolorbutton, *fontentry, *menucheckbutton,
-    *statusbarcheckbutton, *scrollbarcheckbutton;
+    *autologinchannellabel, *autologincommandslabel, *autologinnameentry,
+    *autologinchannelentry, *autologincommandsentry, *fontentry,
+    *menucheckbutton, *statusbarcheckbutton, *scrollbarcheckbutton,
+    *backgroundcolorbutton, *pixmapentry, *pixmapselectbutton,
+    *colorradiobutton, *pixmapradiobutton;
 
+  /* set state of buttons */
   loggingcheckbutton = lookup_widget(widget, "loggingcheckbutton");
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(loggingcheckbutton), 
-    preferences.logging);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(loggingcheckbutton), preferences.logging);
   autologincheckbutton = lookup_widget(widget, "autologincheckbutton");
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autologincheckbutton), 
-    preferences.autologin);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autologincheckbutton), preferences.autologin);
+
+  /* set autologin entries */
   autologinnameentry = lookup_widget(widget, "autologinnameentry");
   autologinchannelentry = lookup_widget(widget, "autologinchannelentry");
+  autologincommandsentry = lookup_widget(widget, "autologincommandsentry");
   if (preferences.name)
     gtk_entry_set_text(GTK_ENTRY(autologinnameentry), preferences.name->str);
   if (preferences.channel)
     gtk_entry_set_text(GTK_ENTRY(autologinchannelentry), 
       preferences.channel->str);
+  if (preferences.commands)
+    gtk_entry_set_text(GTK_ENTRY(autologincommandsentry), 
+      preferences.commands->str);
+
+  /* set sensitivity of widgets in autologin frame */
   if (!preferences.autologin)
   {
     autologinnamelabel = lookup_widget(widget, "autologinnamelabel");
     autologinchannellabel = lookup_widget(widget, "autologinchannellabel");
+    autologincommandslabel = lookup_widget(widget, "autologincommandslabel");
     gtk_widget_set_sensitive(autologinnamelabel, 0);
     gtk_widget_set_sensitive(autologinchannellabel, 0);
+    gtk_widget_set_sensitive(autologincommandslabel, 0);
+    gtk_widget_set_sensitive(autologinnameentry, 0);
+    gtk_widget_set_sensitive(autologinchannelentry, 0);
+    gtk_widget_set_sensitive(autologincommandsentry, 0);
     gtk_editable_set_editable(GTK_EDITABLE(autologinnameentry), FALSE);
     gtk_editable_set_editable(GTK_EDITABLE(autologinchannelentry), FALSE);
+    gtk_editable_set_editable(GTK_EDITABLE(autologincommandsentry), FALSE);
   }
+
+  /* set state of widgets*/
   menucheckbutton = lookup_widget(widget, "menucheckbutton");
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(menucheckbutton), 
-    preferences.menu);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(menucheckbutton), preferences.menu);
   statusbarcheckbutton = lookup_widget(widget, "statusbarcheckbutton");
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(statusbarcheckbutton), 
-    preferences.statusbar);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(statusbarcheckbutton), preferences.statusbar);
   scrollbarcheckbutton = lookup_widget(widget, "scrollbarcheckbutton");
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(scrollbarcheckbutton), 
-    preferences.scrollbar);
-  backgroundcolorbutton = lookup_widget(widget, "backgroundcolorbutton");
-  style = gtk_style_new();
-  style->bg[GTK_STATE_NORMAL] = preferences.background;
-  style->bg[GTK_STATE_PRELIGHT] = preferences.background;
-  gtk_widget_set_style(backgroundcolorbutton, style);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(scrollbarcheckbutton), preferences.scrollbar);
+
+  /* set sensitivity of widgets in background frame*/
+  backgroundcolorbutton = lookup_widget(preferencesdialog, "backgroundcolorbutton");
+  pixmapentry = lookup_widget(preferencesdialog, "pixmapentry");
+  pixmapselectbutton = lookup_widget(preferencesdialog, "pixmapselectbutton");
+  if (preferences.backgroundpixmap) gtk_entry_set_text(GTK_ENTRY(pixmapentry), preferences.backgroundpixmap->str);
+  colorradiobutton = lookup_widget(preferencesdialog, "colorradiobutton");
+  pixmapradiobutton = lookup_widget(preferencesdialog, "pixmapradiobutton");
+  if (preferences.backgroundpixmap) 
+  { 
+  if (preferences.backgroundpixmap->len > 0)
+    {
+      gtk_widget_set_sensitive(backgroundcolorbutton, 0);
+      gtk_widget_set_sensitive(pixmapentry, 1);
+      gtk_widget_set_sensitive(pixmapselectbutton, 1);
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(colorradiobutton), FALSE);
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pixmapradiobutton), TRUE);
+    }
+  }
+  else
+  {
+    gtk_widget_set_sensitive(backgroundcolorbutton, 1);
+    gtk_widget_set_sensitive(pixmapentry, 0);
+    gtk_widget_set_sensitive(pixmapselectbutton, 0);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(colorradiobutton), TRUE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pixmapradiobutton), FALSE);
+  }
+
+  /* update the preview window (argument does not really matter) */
+  updatepreview(preferences.backgroundcolor);
+  
+  /* set colors of all the buttons */
+  setbuttoncolor("backgroundcolorbutton", preferences.backgroundcolor);
+  setbuttoncolor("txmessagebutton", preferences.txfgcolor);
+  setbuttoncolor("rxmessagebutton", preferences.rxfgcolor);
+  setbuttoncolor("statusmessagebutton", preferences.statusfgcolor);
+  setbuttoncolor("privatemessagebutton", preferences.privatefgcolor);
+  setbuttoncolor("user1button", preferences.userfgcolor[0]);
+  setbuttoncolor("user2button", preferences.userfgcolor[1]);
+  setbuttoncolor("user3button", preferences.userfgcolor[2]);
+  setbuttoncolor("user4button", preferences.userfgcolor[3]);
+  setbuttoncolor("user5button", preferences.userfgcolor[4]);
+  setbuttoncolor("user6button", preferences.userfgcolor[5]);
+  setbuttoncolor("user7button", preferences.userfgcolor[6]);
+  setbuttoncolor("user8button", preferences.userfgcolor[7]);
+  setbuttoncolor("user9button", preferences.userfgcolor[8]);
+  setbuttoncolor("user10button", preferences.userfgcolor[9]);
+
+  /* find font and set it */
   fontentry = lookup_widget(widget, "fontentry");
   gtk_entry_set_text(GTK_ENTRY(fontentry), preferences.font->str);
+
+  /* finally, make the main window insensitive */
   gtk_widget_set_sensitive(mainwindow, 0);
 }
 
@@ -473,59 +509,130 @@ void on_preferencesdialog_show(GtkWidget *widget, gpointer user_data)
 void on_autologincheckbutton_toggled(GtkToggleButton *togglebutton,
   gpointer user_data)
 {
-  GtkWidget *autologinnamelabel, *autologinchannellabel, *autologinnameentry,
-    *autologinchannelentry;
+  GtkWidget *autologinnamelabel, *autologinchannellabel,
+    *autologincommandslabel,*autologinnameentry, *autologinchannelentry, 
+    *autologincommandsentry;
   gboolean autologincheckbuttonstatus;
 
   autologincheckbuttonstatus = gtk_toggle_button_get_active
     (GTK_TOGGLE_BUTTON(togglebutton));
   preferencesdialog = gtk_widget_get_toplevel(GTK_WIDGET(togglebutton));
   autologinnamelabel = lookup_widget(preferencesdialog, "autologinnamelabel");
-  autologinchannellabel = lookup_widget(preferencesdialog, 
-    "autologinchannellabel");
+  autologinchannellabel = lookup_widget(preferencesdialog, "autologinchannellabel");
+  autologincommandslabel = lookup_widget(preferencesdialog, "autologincommandslabel");
   autologinnameentry = lookup_widget(preferencesdialog, "autologinnameentry");
-  autologinchannelentry = lookup_widget(preferencesdialog, 
-    "autologinchannelentry");
-  if (autologincheckbuttonstatus)
+  autologinchannelentry = lookup_widget(preferencesdialog, "autologinchannelentry");
+  autologincommandsentry = lookup_widget(preferencesdialog, "autologincommandsentry");
+  gtk_widget_set_sensitive(autologinnamelabel, autologincheckbuttonstatus);
+  gtk_widget_set_sensitive(autologinchannellabel, autologincheckbuttonstatus);
+  gtk_widget_set_sensitive(autologincommandslabel, autologincheckbuttonstatus);
+  gtk_widget_set_sensitive(autologinnameentry, autologincheckbuttonstatus);
+  gtk_widget_set_sensitive(autologinchannelentry, autologincheckbuttonstatus);
+  gtk_widget_set_sensitive(autologincommandsentry, autologincheckbuttonstatus);
+  gtk_editable_set_editable(GTK_EDITABLE(autologinnameentry),
+    autologincheckbuttonstatus);
+  gtk_editable_set_editable(GTK_EDITABLE(autologinchannelentry),
+    autologincheckbuttonstatus);
+  gtk_editable_set_editable(GTK_EDITABLE(autologincommandsentry),
+    autologincheckbuttonstatus);
+}
+
+/*
+ * We look up the color of the clicked button and apply this to the
+ * colorselection dialog. The preferences dialog is made insensitive and the
+ * colorselectiondialog is shown. I use the button_pressed_event here because
+ * I want to distinguish between left and right mouse clicks.
+ */
+
+gboolean
+on_colorbutton_pressed_event           (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data)
+{
+  GtkWidget *colorselectiondialog;
+  gdouble rgb[3];
+  GdkColor buttoncolor;
+  GtkStyle *buttonstyle;
+
+  /* Save button and mouse button pressed */
+  buttonnumber = atoi(user_data);
+  mousebutton = event->button;
+
+  /* Stop the event */
+  gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event");
+
+  colorselectiondialog = create_colorselectiondialog();
+  gtk_widget_destroy(GTK_COLOR_SELECTION_DIALOG(colorselectiondialog)->help_button);
+
+  /* Get the buttoncolor before applying it to the color selection dialog. */
+  /*Special case for private color button: right-click should select the background color*/
+  buttonstyle = gtk_widget_get_style(widget);
+  if (!(buttonnumber == 4)) buttoncolor = buttonstyle->bg[GTK_STATE_NORMAL];
+    else 
+    {
+      if (mousebutton == 1) buttoncolor = preferences.privatefgcolor;
+      else buttoncolor = preferences.privatebgcolor;
+    }
+
+  rgb[0] = buttoncolor.red / 65535.0;
+  rgb[1] = buttoncolor.green / 65535.0;
+  rgb[2] = buttoncolor.blue / 65535.0;
+  gtk_color_selection_set_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(colorselectiondialog)->colorsel), rgb);
+  gtk_widget_set_sensitive(preferencesdialog, 0);
+  gtk_widget_show(colorselectiondialog);
+
+  return FALSE;
+}
+
+/*
+ * Either background color or pixmap is selected here. Set the sensitivity of
+ * the associated widgets accordingly.
+ */
+
+void on_radiobutton_toggled (GtkToggleButton *togglebutton, gpointer user_data)
+{
+  gint data;
+  GtkWidget *backgroundcolorbutton, *pixmapentry, *pixmapselectbutton;
+
+  data = atoi(user_data);
+  backgroundcolorbutton = lookup_widget(preferencesdialog, "backgroundcolorbutton");
+  pixmapentry = lookup_widget(preferencesdialog, "pixmapentry");
+  pixmapselectbutton = lookup_widget(preferencesdialog, "pixmapselectbutton");
+
+  switch (data)
   {
-    gtk_widget_set_sensitive(autologinnamelabel, 1);
-    gtk_widget_set_sensitive(autologinchannellabel, 1);
-    gtk_widget_set_sensitive(autologinnameentry, 1);
-    gtk_widget_set_sensitive(autologinchannelentry, 1);
-    gtk_editable_set_editable(GTK_EDITABLE(autologinnameentry), TRUE);
-    gtk_editable_set_editable(GTK_EDITABLE(autologinchannelentry), TRUE);
-  }
-  else
-  {
-    gtk_widget_set_sensitive(autologinnamelabel, 0);
-    gtk_widget_set_sensitive(autologinchannellabel, 0);
-    gtk_widget_set_sensitive(autologinnameentry, 0);
-    gtk_widget_set_sensitive(autologinchannelentry, 0);
-    gtk_editable_set_editable(GTK_EDITABLE(autologinnameentry), FALSE);
-    gtk_editable_set_editable(GTK_EDITABLE(autologinchannelentry), FALSE);
+    case 1:
+      gtk_widget_set_sensitive(backgroundcolorbutton, 1);
+      gtk_widget_set_sensitive(pixmapentry, 0);
+      gtk_widget_set_sensitive(pixmapselectbutton, 0);
+    break;
+    case 2:
+      gtk_widget_set_sensitive(backgroundcolorbutton, 0);
+      gtk_widget_set_sensitive(pixmapentry, 1);
+      gtk_widget_set_sensitive(pixmapselectbutton, 1);
+    break;
+    default:
+    break;
   }
 }
 
 /*
- * We want to change the background color of the main window, so we look up the
- * color and apply this to the color selection dialog.
+ * Select a background pixmap. Show the openfile dialog.
  */
 
-void on_backgroundcolorbutton_clicked (GtkButton *button, gpointer user_data)
+void on_pixmapselectbutton_clicked (GtkButton *button, gpointer user_data)
 {
-  GtkWidget *colorselectiondialog;
-  gdouble rgb[3];
+  GtkWidget *fileselectiondialog;
+  
+  fileselectiondialog = create_fileselectiondialog();
+  
+  /* Don't need these buttons */
+  gtk_widget_destroy(GTK_FILE_SELECTION(fileselectiondialog)->fileop_c_dir);
+  gtk_widget_destroy(GTK_FILE_SELECTION(fileselectiondialog)->fileop_del_file);
+  gtk_widget_destroy(GTK_FILE_SELECTION(fileselectiondialog)->fileop_ren_file);
 
-  colorselectiondialog = create_colorselectiondialog();
-  gtk_widget_destroy(GTK_COLOR_SELECTION_DIALOG(colorselectiondialog )->
-    help_button);
-  rgb[0] = preferences.background.red / 65535.0;
-  rgb[1] = preferences.background.green / 65535.0;
-  rgb[2] = preferences.background.blue / 65535.0;
-  gtk_color_selection_set_color(GTK_COLOR_SELECTION(
-    GTK_COLOR_SELECTION_DIALOG(colorselectiondialog)->colorsel), rgb);
   gtk_widget_set_sensitive(preferencesdialog, 0);
-  gtk_widget_show(colorselectiondialog);
+  gtk_widget_show(fileselectiondialog);
 }
 
 /*
@@ -538,10 +645,8 @@ void on_fontbutton_clicked (GtkButton *button, gpointer user_data)
   GtkWidget *fontselectiondialog;
 
   fontselectiondialog = create_fontselectiondialog();
-  gtk_font_selection_dialog_set_preview_text(GTK_FONT_SELECTION_DIALOG
-    (fontselectiondialog), "How about this font?");
-  gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG
-    (fontselectiondialog), preferences.font->str);
+  gtk_font_selection_dialog_set_preview_text(GTK_FONT_SELECTION_DIALOG(fontselectiondialog), _("How about this font?"));
+  gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(fontselectiondialog), preferences.font->str);
   gtk_widget_set_sensitive(preferencesdialog, 0);
   gtk_widget_show(fontselectiondialog);
 }
@@ -556,72 +661,91 @@ void on_fontbutton_clicked (GtkButton *button, gpointer user_data)
 void on_preferencesOKbutton_clicked(GtkButton *button, gpointer user_data)
 {
   GtkWidget *loggingcheckbutton, *autologincheckbutton, *autologinnameentry,
-    *autologinchannelentry, *maintext, *menucheckbutton, *statusbarcheckbutton,
-    *scrollbarcheckbutton, *mainscrolledwindow;
-  gboolean loggingcheckbuttonstatus, autologincheckbuttonstatus,
-    menucheckbuttonstatus, statusbarcheckbuttonstatus,
-    scrollbarcheckbuttonstatus;
+    *autologinchannelentry, *autologincommandsentry, *maintext,
+    *menucheckbutton, *statusbarcheckbutton, *scrollbarcheckbutton, 
+    *mainscrolledwindow, *colorradiobutton;
   GtkStyle *maintextstyle;
+  GdkPixmap *pixmap = NULL;
+  GdkBitmap *mask;
+  gboolean coloryes;
 
+  /* Save state of logging */
   loggingcheckbutton = lookup_widget(preferencesdialog, "loggingcheckbutton");
-  loggingcheckbuttonstatus = gtk_toggle_button_get_active
+  preferences.logging = gtk_toggle_button_get_active
     (GTK_TOGGLE_BUTTON(loggingcheckbutton));
-  preferences.logging = loggingcheckbuttonstatus;
+
+  /* Save the entries for autologin */
   autologincheckbutton = lookup_widget(preferencesdialog, 
     "autologincheckbutton");
-  autologincheckbuttonstatus = gtk_toggle_button_get_active
+  preferences.autologin = gtk_toggle_button_get_active
     (GTK_TOGGLE_BUTTON(autologincheckbutton));
-  preferences.autologin = autologincheckbuttonstatus;
   if (preferences.autologin)
   {
-    autologinnameentry = lookup_widget(preferencesdialog, 
-      "autologinnameentry");
-    autologinchannelentry = lookup_widget(preferencesdialog, 
-      "autologinchannelentry");
-    preferences.name = g_string_new(gtk_editable_get_chars(GTK_EDITABLE
-      (autologinnameentry), 0, -1));
-    preferences.channel = g_string_new(gtk_editable_get_chars(GTK_EDITABLE
-      (autologinchannelentry), 0, -1));
+    autologinnameentry = lookup_widget(preferencesdialog, "autologinnameentry");
+    autologinchannelentry = lookup_widget(preferencesdialog, "autologinchannelentry");
+    autologincommandsentry = lookup_widget(preferencesdialog, "autologincommandsentry");
+    preferences.name = g_string_new(gtk_editable_get_chars(GTK_EDITABLE(autologinnameentry), 0, -1));
+    preferences.channel = g_string_new(gtk_editable_get_chars(GTK_EDITABLE(autologinchannelentry), 0, -1));
+    preferences.commands = g_string_new(gtk_editable_get_chars(GTK_EDITABLE(autologincommandsentry), 0, -1));
   }
+
+  /* Save visibility of some widgets */
   menucheckbutton = lookup_widget(preferencesdialog, "menucheckbutton");
-  menucheckbuttonstatus = gtk_toggle_button_get_active
-    (GTK_TOGGLE_BUTTON(menucheckbutton));
-  widget_visible("mainmenubar", menucheckbuttonstatus);
-  preferences.menu = menucheckbuttonstatus;
-  statusbarcheckbutton = lookup_widget(preferencesdialog,
-    "statusbarcheckbutton");
-  statusbarcheckbuttonstatus = gtk_toggle_button_get_active
-    (GTK_TOGGLE_BUTTON(statusbarcheckbutton));
-  widget_visible("mainstatusbar", statusbarcheckbuttonstatus);
-  preferences.statusbar = statusbarcheckbuttonstatus;
-  scrollbarcheckbutton = lookup_widget(preferencesdialog,
-    "scrollbarcheckbutton");
-  scrollbarcheckbuttonstatus = gtk_toggle_button_get_active
-    (GTK_TOGGLE_BUTTON(scrollbarcheckbutton));
-  preferences.scrollbar = scrollbarcheckbuttonstatus;
+  preferences.menu = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(menucheckbutton));
+  widget_visible("mainmenubar", preferences.menu);
+  statusbarcheckbutton = lookup_widget(preferencesdialog, "statusbarcheckbutton");
+  preferences.statusbar = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(statusbarcheckbutton));
+  widget_visible("mainstatusbar", preferences.statusbar);
+  scrollbarcheckbutton = lookup_widget(preferencesdialog, "scrollbarcheckbutton");
+  preferences.scrollbar = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(scrollbarcheckbutton));
   mainscrolledwindow = lookup_widget(mainwindow, "mainscrolledwindow");
-  if (scrollbarcheckbuttonstatus)
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(mainscrolledwindow), 
-      GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+  if (preferences.scrollbar)
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(mainscrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
   else
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(mainscrolledwindow), 
-      GTK_POLICY_NEVER, GTK_POLICY_NEVER);
-  savepreferences();
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(mainscrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
+
+  /* Check if we use a pixmap. If yes, see if we can create it. If we can't, revert to background color */
+  colorradiobutton = lookup_widget(preferencesdialog, "colorradiobutton");
+  coloryes = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(colorradiobutton)));
+  maintext = lookup_widget(mainwindow, "maintext");
+  if (!(coloryes))
+  {
+    pixmap = gdk_pixmap_create_from_xpm(maintext->window, &mask, NULL, preferences.backgroundpixmap->str);
+    if (!(pixmap)) coloryes = TRUE;
+  }
+
+  savepreferences(coloryes);
+
+  /* Turn logging on/off if it is set/unset and we are connected */
   if (preferences.logging && connected)
   {
     logfilename = g_strconcat(preferencesdir, "/", cvhost, ".log", NULL);
     if ((logfp = fopen(logfilename, "w")) == NULL) 
-      g_error("Creating %s for writing.", logfilename);
+      g_error(_("Creating %s for writing."), logfilename);
   }
   else 
   { 
-    if (!preferences.logging && connected) fclose(logfp);
+    if (!preferences.logging && connected) if (logfp != NULL) fclose(logfp);
   }
-  maintext = lookup_widget(mainwindow, "maintext");
-  maintextstyle = gtk_style_new();
-  maintextstyle->base[GTK_STATE_NORMAL] = preferences.background;
-  maintextstyle->base[GTK_STATE_INSENSITIVE] = preferences.background;
-  gtk_widget_set_style(maintext, maintextstyle);
+
+  /* Change the background. Either make it a color or a pixmap */
+  if (coloryes)
+  {
+    maintextstyle = gtk_style_new();
+    maintextstyle->base[GTK_STATE_NORMAL] = preferences.backgroundcolor;
+    maintextstyle->base[GTK_STATE_INSENSITIVE] = preferences.backgroundcolor;
+    gtk_widget_set_style(maintext, maintextstyle);
+    preferences.backgroundpixmap = g_string_new("");
+  }
+  else
+  {
+    maintextstyle = gtk_style_copy(maintext->style);
+    gtk_style_ref(maintextstyle);
+    maintextstyle->bg_pixmap[GTK_STATE_NORMAL] = pixmap;
+    gtk_widget_set_style(maintext, maintextstyle);
+    gtk_style_unref(maintextstyle);
+  }
+
   gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET (button)));
 }
 
@@ -651,43 +775,31 @@ void on_preferencesdialog_destroy(GtkObject *object, gpointer user_data)
 
 /*
  * Update the about dialog with some information before it is shown.
- * Title of about window is set here too.
  */
 
 void on_aboutdialog_show(GtkWidget *widget, gpointer user_data)
 {
-  GString *info;
+  GString *info = g_string_new("");
   GtkWidget *aboutcreditslabel, *aboutlicenselabel, *abouthelptext;
 
   aboutcreditslabel = lookup_widget(widget, "aboutcreditslabel");
   abouthelptext = lookup_widget(widget, "abouthelptext");
   aboutlicenselabel = lookup_widget(widget, "aboutlicenselabel");
-  info = g_string_new("\n");
-  g_string_append(info, PACKAGE);
-  g_string_append(info, " version ");
-  g_string_append(info, VERSION);
-  g_string_append(info, "\nCopyright (C) 2000-2001 Joop Stakenborg\n");
-  g_string_append(info, "E-mail: <pa4tu@amsat.org>\n");
-  g_string_append(info, "http://xconvers.sourceforge.net\n\n");
+
+  g_string_sprintf(info, _("\n%s version %s\nCopyright (C) 2000-2001 Joop Stakenborg\n\
+E-mail: <pa4tu@amsat.org>\nhttp://xconvers.sourceforge.net\n\n"), PACKAGE, VERSION);
   gtk_label_set_text(GTK_LABEL(aboutcreditslabel), info->str);
-  g_string_free(info, TRUE);
-  info = g_string_new("");
-  g_string_append(info, "PgUp: Scroll the receive window up.\n");
-  g_string_append(info, "PgDn: Scroll the receive window down.\n");
-  g_string_append(info, "Up Arrow: Recall previously transmitted text.\n");
-  g_string_append(info, "Dn Arrow: Recall next transmitted text.\n\n");
-  g_string_append(info, "Alt+F: File menu (exit program).\n");
-  g_string_append(info, "Alt+H: Host menu (open/close a connection).\n");
-  g_string_append(info, "Alt+S: Settings menu (preferences).\n");
-  g_string_append(info, "Alt+P: Help menu (credits and help).\n");
+
+  g_string_sprintf(info, _("PgUp: Scroll the receive window up.\nPgDn: Scroll the receive window down.\n\
+Up Arrow: Recall previously transmitted text.\nDn Arrow: Recall next transmitted text.\n\n\
+Alt+F: File menu (exit program).\nAlt+H: Host menu (open/close a connection).\n\
+Alt+S: Settings menu (preferences).\nAlt+P: Help menu (credits and help).\n"));
   gtk_text_insert(GTK_TEXT(abouthelptext), NULL, NULL, NULL, info->str, -1);
-  g_string_free(info, TRUE);
-  info = g_string_new("\n");
-  g_string_append(info, "Xconvers comes with ABSOLUTELY NO WARRANTY.\n");
-  g_string_append(info, "This is free software, and you are welcome to\n"); 
-  g_string_append(info, "redistribute it under certain conditions.\n");
-  g_string_append(info, "For details see the GNU General Public License.\n");
+
+  g_string_sprintf(info, _("Xconvers comes with ABSOLUTELY NO WARRANTY.\nThis is free software, and you are welcome to\n\
+redistribute it under certain conditions.\nFor details see the GNU General Public License.\n"));
   gtk_label_set_text(GTK_LABEL(aboutlicenselabel), info->str);
+
   g_string_free(info, TRUE);
 }
 
@@ -707,29 +819,23 @@ void on_aboutOKbutton_clicked(GtkButton *button, gpointer user_data)
 /************************** COLOR SELECTION DIALOG ***************************/
 
 /*
- * We have selected a color, so now we can set the color of the button on the 
- * preferences dialog.
+ * We have selected a color, so now we can set the color of the button in the 
+ * preferences dialog and update the preview window.
  */
 
 void on_colorok_button_clicked(GtkButton *button, gpointer user_data)
 {
-  GtkWidget *colorselectiondialog, *backgroundcolorbutton;
-  GtkStyle *backgroundcolorbuttonstyle;
+  GtkWidget *colorselectiondialog;
+  GdkColor newcolor;
   gdouble rgb[3];
   
-  backgroundcolorbuttonstyle = gtk_style_new();
   colorselectiondialog = gtk_widget_get_toplevel(GTK_WIDGET(button));
-  gtk_color_selection_get_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG
-    (colorselectiondialog)->colorsel), rgb);
-  preferences.background.red = rgb[0] * 65535.0;
-  preferences.background.green = rgb[1] * 65535.0;
-  preferences.background.blue = rgb[2] * 65535.0;
-  backgroundcolorbuttonstyle->bg[GTK_STATE_NORMAL] = preferences.background;
-  backgroundcolorbuttonstyle->bg[GTK_STATE_PRELIGHT] = preferences.background;
-  backgroundcolorbutton = lookup_widget(preferencesdialog,
-    "backgroundcolorbutton");
-  gtk_widget_set_style(backgroundcolorbutton, backgroundcolorbuttonstyle);
+  gtk_color_selection_get_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(colorselectiondialog)->colorsel), rgb);
+  newcolor.red = rgb[0] * 65535.0;
+  newcolor.green = rgb[1] * 65535.0;
+  newcolor.blue = rgb[2] * 65535.0;
   gtk_widget_set_sensitive(preferencesdialog, 1);
+  updatepreview(newcolor);
   gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
 }
 
@@ -746,8 +852,8 @@ void on_colorcancel_button_clicked (GtkButton *button, gpointer user_data)
 /************************* FONT SELECTION DIALOG *****************************/
 
 /*
- * A font is selected. Update the preferences struct and change the text font
- * for the main window.
+ * A font is selected. Update the preferences struct, change the text font
+ * for the main window and change the preview window.
  */
 
 void on_fontok_button_clicked (GtkButton *button, gpointer user_data)
@@ -755,11 +861,11 @@ void on_fontok_button_clicked (GtkButton *button, gpointer user_data)
   GtkWidget *fontselectiondialog, *fontentry;
   
   fontselectiondialog = gtk_widget_get_toplevel(GTK_WIDGET(button));
-  preferences.font = g_string_new(gtk_font_selection_dialog_get_font_name
-    (GTK_FONT_SELECTION_DIALOG(fontselectiondialog)));
+  preferences.font = g_string_new(gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(fontselectiondialog)));
   fontentry = lookup_widget(preferencesdialog, "fontentry");
   gtk_entry_set_text(GTK_ENTRY(fontentry), preferences.font->str);
   textfont = gdk_font_load(preferences.font->str);
+  updatepreview(preferences.backgroundcolor);
   gtk_widget_set_sensitive(preferencesdialog, 1);
   gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
 }
@@ -773,3 +879,34 @@ void on_fontcancel_button_clicked (GtkButton *button, gpointer user_data)
   gtk_widget_set_sensitive(preferencesdialog, 1);
   gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
 }
+
+/************************* FILE SELECTION DIALOG *****************************/
+
+/*
+ * Select a pixmap for the text background and fill the entry in the
+ * preferences dialog.
+ */
+
+void
+on_fileok_button_clicked (GtkButton *button, gpointer user_data)
+{
+  GtkWidget *fileselectiondialog, *pixmapentry;
+  
+  fileselectiondialog = gtk_widget_get_toplevel(GTK_WIDGET(button));
+  preferences.backgroundpixmap = g_string_new(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselectiondialog)));
+  pixmapentry = lookup_widget(preferencesdialog, "pixmapentry");
+  gtk_entry_set_text(GTK_ENTRY(pixmapentry), preferences.backgroundpixmap->str);
+  gtk_widget_set_sensitive(preferencesdialog, 1);
+  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
+}
+
+/*
+ * Just destroy the dialog when cancel is clicked.
+ */
+
+void
+on_filecancel_button_clicked (GtkButton *button, gpointer user_data)
+{
+  gtk_widget_set_sensitive(preferencesdialog, 1);
+  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
+}
