@@ -31,5 +31,13 @@ typedef struct
   gboolean statusbar;
   gboolean scrollbar;
   GString *font;
-  GdkColor background;
+  GString *commands;
+  GString *backgroundpixmap;
+  GdkColor backgroundcolor;
+  GdkColor txfgcolor;
+  GdkColor rxfgcolor;
+  GdkColor statusfgcolor;
+  GdkColor privatefgcolor;
+  GdkColor privatebgcolor;
+  GdkColor userfgcolor[10];
 } preferencestype;
\ No newline at end of file
